"use strict";

var _logger = require("../logger");

var _SourceMapUrls = require("./SourceMapUrls");

/*
  This is a file used to get all the sources maps of a project.
  It is based on a deprecated library called `source-map-resolve`.
  `SourceMapUrls.js` is a slimmer version of said library.
  We had to change the library because we ran into a problem where it didn't get all source maps if their URLs were found in the same file.
*/
var innerRegex = /[#@] sourceMappingURL=([^\s'"]*)/;
var sourceMappingURLRegex = RegExp("(?:" + "/\\*" + "(?:\\s*\r?\n(?://)?)?" + "(?:" + innerRegex.source + ")" + "\\s*" + "\\*/" + "|" + "//(?:" + innerRegex.source + ")" + ")" + "\\s*", "g" // This is an important flag added to allow multiple regex matches
);
var dataUriRegex = /^data:([^,;]*)(;[^,;]*)*(?:,(.*))?$/;
/**
 * The media type for JSON text is application/json.
 *
 * {@link https://tools.ietf.org/html/rfc8259#section-11 | IANA Considerations }
 *
 * `text/json` is non-standard media type
 */

var jsonMimeTypeRegex = /^(?:application|text)\/json$/; // A function to get all URLs from a single file

function getAllUrls(code) {
  var match;
  const allSourcesUrl = [];

  do {
    match = sourceMappingURLRegex.exec(code);

    if (match) {
      allSourcesUrl.push(match[1] || match[2] || "");
    }
  } while (match);

  return allSourcesUrl;
} // Creates a new object containing the relevant data for a source map


function parseSourceMappingUrl(url, codeUrl) {
  codeUrl = (0, _SourceMapUrls.convertWindowsPath)(codeUrl);

  if (!url) {
    return null;
  } // inline sources


  var dataUri = url.match(dataUriRegex);

  if (dataUri) {
    var mimeType = dataUri[1] || "text/plain";
    var lastParameter = dataUri[2] || "";
    var encoded = dataUri[3] || "";
    var data = {
      sourceMappingURL: url,
      url: null,
      sourcesRelativeTo: codeUrl,
      map: encoded
    };

    if (!jsonMimeTypeRegex.test(mimeType)) {
      var error = new Error("Unuseful data uri mime type: " + mimeType);
      error.sourceMapData = data;
      throw error;
    }

    try {
      data.map = (0, _SourceMapUrls.parseMapToJSON)(lastParameter === ";base64" ? (0, _SourceMapUrls.decodeBase64String)(encoded) : decodeURIComponent(encoded), data);
    } catch (error) {
      error.sourceMapData = data;
      throw error;
    }

    return data;
  } // sources in different file


  var mapUrl = (0, _SourceMapUrls.resolveUrl)(codeUrl, url);
  return {
    sourceMappingURL: url,
    url: mapUrl,
    sourcesRelativeTo: mapUrl,
    map: null
  };
} // Returns a list of objects containing the source maps' data


function allSourceMaps(code, codeUrl, read) {
  const allSources = [];
  var match = getAllUrls(code);

  for (let sourceMap of match) {
    _logger.logger.debug("Parsing sourceMapURL: ", sourceMap);

    try {
      var mapData = parseSourceMappingUrl(sourceMap, codeUrl);
    } catch (error) {
      _logger.logger.debug("Skipping this sourceMapURL: ", sourceMap, ". ", error, " was thrown.");

      continue;
    } // if the source map is in a different file we need to read it


    if (mapData && !mapData.map) {
      try {
        mapData.map = (0, _SourceMapUrls.readSync)(read, mapData.url, mapData);
        mapData.map = (0, _SourceMapUrls.parseMapToJSON)(mapData.map, mapData);
      } catch (error) {
        _logger.logger.debug("Failed to read and parse the source map (", mapData.url, "), skipping. ", error, " was thrown.");

        continue;
      }
    }

    if (mapData) {
      allSources.push(mapData);
    }
  }

  return allSources;
}

module.exports = {
  allSourceMaps: allSourceMaps
};
//# sourceMappingURL=SourceMapsHelper.js.map